@@ -1,119 +1,105 @@
-package Thread::Serialize;
+use 5.014;
+#package Thread::Serialize '1.00'; # not supported by PAUSE or MetaCPAN :-(
+package Thread::Serialize;         # please remove if no longer needed
 
-# Make sure we have version info for this module
-# Make sure we do everything by the book from now on
+our $VERSION= '1.00';              # please remove if no longer needed
 
-$VERSION = '0.11';
-use strict;
-
-# Make sure we only load things that we need when we need it
-
-use load;
-
-# Make sure we can freeze and thaw
+# be as verbose as possble
+use warnings;;
 
+# modules that we need
 use Storable ();
 
-# Initialize Storable signature
-# If we should not use an external Perl to determine signature
-#  Use local copy of routines, causing AutoLoader to load lot's of stuff
-# Else
-#  Execute external perl to obtain Storable signature (saves memory here)
-
+# use ourselves to determine signature
 our $iced;
-if ($Thread::Serialize::no_external_perl) {
-    $iced = unpack( 'l',Storable::freeze( [] ));
-    $Thread::Serialize::no_external_perl = 'Signature obtained locally';
-} else {
-    open( my $handle,
-     qq($^X -MStorable -e "print unpack('l',Storable::freeze( [] ))" | )
-    ) or die "Cannot determine Storable signature\n";
-    $iced = <$handle>;
+if ( $Thread::Serialize::no_external_perl ) {
+    $iced= unpack( 'l',Storable::freeze( [] ));
+    $Thread::Serialize::no_external_perl= 'Signature obtained locally';
 }
 
-# Satisfy -require-
+# use external perl to create signature
+else {
+    open( my $handle,
+      qq( $^X -MStorable -e "print unpack( 'l', Storable::freeze( [] ) )" | )
+    ) or die "Cannot determine Storable signature\n";
+    $iced= readline $handle;
+}
 
+# satisfy -require-
 1;
 
-# The following subroutines are loaded on demand only
-
-__END__
-
-#---------------------------------------------------------------------------
+#-------------------------------------------------------------------------------
+# freeze
+#
+# Freeze given parameters into a single scalar value
+#
 #  IN: 1..N parameters to freeze
 # OUT: 1 frozen scalar
 
 sub freeze {
 
-# If we have at least one element in the list
-#  For all of the elements
-#   Return truly frozen version if something special
-#  Return the values contatenated with null bytes
-# Else (empty list)
-#  Return undef value
-
+    # something to be frozen
     if (@_) {
+
+        # any of the parameters is special, so really freeze
         foreach (@_) {
             return Storable::freeze( \@_ ) if !defined() or ref() or m#\0#;
         }
-        return join( "\0",@_ );
-    } else {
-        return;
+
+        # just concatenate with null bytes (WAY faster)
+        return join( "\0", @_ );
     }
+
+    # no parameters, so just undef
+    return;
 } #freeze
 
-#---------------------------------------------------------------------------
+#-------------------------------------------------------------------------------
 #  IN: 1 frozen scalar to defrost
 # OUT: 1..N thawed data structure
 
 sub thaw {
 
-# Return now if nothing to return or not interested in result
-
-    return unless defined( $_[0] ) and defined( wantarray );
-
-# If we're interested in a list
-#  Return thawed list from frozen info if frozen
-#  Return list split from a normal string
-# Elseif we have frozen stuff (and we want a scalar)
-#  Thaw the list and return the first element
-# Else (not frozen and we want a scalar)
-#  Look for the first nullbyte and return string until then if found
-#  Return the string
+    # nothing here or not interested
+    return if !defined( $_[0] ) or !defined(wantarray);
 
+    # return as list
     if (wantarray) {
-        return @{Storable::thaw( $_[0] )} if (unpack( 'l',$_[0] )||0) == $iced;
-        split( "\0",$_[0] )
-    } elsif ((unpack( 'l',$_[0] )||0) == $iced) {
-        Storable::thaw( $_[0] )->[0];
-    } else {
-	return $1 if $_[0] =~ m#^([^\0]*)#;
-        $_[0];
+        return ( unpack( 'l', $_[0] ) || 0 ) == $iced
+          ? @{ Storable::thaw( $_[0] ) }
+          : split "\0", $_[0];
     }
+
+    # scalar, really frozen, return first value
+    elsif ( ( unpack( 'l',$_[0] ) || 0 ) == $iced ) {
+        return Storable::thaw( $_[0] )->[0];
+    }
+
+    # return first part of scalar
+    return $_[0] =~ m#^([^\0]*)# ? $1 : $_[0];
 } #thaw
 
-#---------------------------------------------------------------------------
-
-# standard Perl features
-
-#---------------------------------------------------------------------------
+#-------------------------------------------------------------------------------
+#
+# Standard Perl Features
+#
+#-------------------------------------------------------------------------------
 
 sub import {
-
-# Lose the class
-# Obtain the namespace
-# Obtain the names of the subroutines to export
-# Allow for dirty tricks
-# Export all subroutines specified
-
     shift;
-    my $namespace = caller().'::';
-    @_ = qw(freeze thaw) unless @_;
+
+    # determine namespace and subs
+    my $namespace= caller().'::';
+    @_= qw( freeze thaw ) if !@_;
+
+    # do our exports
     no strict 'refs';
-    *{$namespace.$_} = \&$_ foreach @_;
+    *{$namespace.$_}= \&$_ foreach @_;
+
+    return;
 } #import
 
-#---------------------------------------------------------------------------
+#-------------------------------------------------------------------------------
 
 __END__
 
@@ -132,16 +118,14 @@ Thread::Serialize - serialize data-structures between threads
 
 =head1 VERSION
 
-This documentation describes version 0.11.
+This documentation describes version 1.00.
 
 =head1 DESCRIPTION
 
                   *** A note of CAUTION ***
 
- This module only functions on Perl versions 5.8.0 and later.
- And then only when threads are enabled with -Dusethreads.  It
- is of no use with any version of Perl before 5.8.0 or without
- threads enabled.
+ This module only functions if threading has been enabled when building
+ Perl, or if the "forks" module has been installed on an unthreaded Perl.
 
                   *************************
 
@@ -182,8 +166,35 @@ are always matched by scalar context calls to L<thaw>.
 
 =head1 REQUIRED MODULES
 
- load (0.10)
  Storable (any)
+ Test::More (0.88)
+
+=head1 INSTALLATION
+
+This distribution contains two versions of the code: one maintenance version
+for versions of perl < 5.014 (known as 'maint'), and the version currently in
+development (known as 'blead').  The standard build for your perl version is:
+
+ perl Makefile.PL
+ make
+ make test
+ make install
+
+This will try to test and install the "blead" version of the code.  If the
+Perl version does not support the "blead" version, then the running of the
+Makefile.PL will *fail*.  In such a case, one can force the installing of
+the "maint" version of the code by doing:
+
+ perl Makefile.PL maint
+
+Alternately, if you want automatic selection behavior, you can set the
+AUTO_SELECT_MAINT_OR_BLEAD environment variable to a true value.  On Unix-like
+systems like so:
+
+ AUTO_SELECT_MAINT_OR_BLEAD=1 perl Makefile.PL
+
+If your perl does not support the "blead" version of the code, then it will
+automatically install the "maint" version of the code.
 
 =head1 OPTIMIZATIONS
 
@@ -200,7 +211,7 @@ In some situations this may cause a problem: please set the
 C<$Thread::Serialize::no_external_perl> variable to a true value at compile
 time B<before> loading Thread::Serialize if this causes a problem.
 
- BEGIN { $Thread::Serialize::no_external_perl = 1 }
+ BEGIN { $Thread::Serialize::no_external_perl= 1 }
  use Thread::Serialize;
 
 =head1 KNOWN ISSUES
@@ -220,9 +231,9 @@ Please report bugs to <perlbugs@dijkmat.nl>.
 
 =head1 COPYRIGHT
 
-Copyright (c) 2002-2004, 2010 Elizabeth Mattijsen <liz@dijkmat.nl>. All rights
-reserved.  This program is free software; you can redistribute it and/or
-modify it under the same terms as Perl itself.
+Copyright (c) 2002, 2003, 2004, 2010, 2012 Elizabeth Mattijsen <liz@dijkmat.nl>.
+All rights reserved.  This program is free software; you can redistribute it
+and/or modify it under the same terms as Perl itself.
 
 =head1 SEE ALSO
 
